@@ -2,6 +2,7 @@ import json
 import logging
 import sys
 import uuid
+from autobahn.twisted.util import sleep
 from confire import Configuration
 
 import re
@@ -22,11 +23,7 @@ class ApiRpcClient(object):
         self.host = host
         self.port = port
         self.settings = settings
-
-    @inlineCallbacks
-    def start_task(self, task, **kwargs):
-        yield getattr(self, task)(**kwargs)
-        reactor.stop()
+        self.stop_reactor = True
 
     @inlineCallbacks
     def _remote_exec(self, task_name, on_result, *args, **kwargs):
@@ -52,6 +49,10 @@ class ApiRpcClient(object):
             print 'Can\'t connect to mfcloud server'
 
         client.shutdown()
+        yield sleep(0.01)
+
+        if self.stop_reactor:
+            reactor.stop()
 
     #
     #
@@ -127,12 +128,12 @@ class ApiRpcClient(object):
         self._remote_exec('init', self.on_print_list_result, name, os.path.realpath(path))
 
 
-    def on_print_list_result(self, data):
+    def on_print_list_result(self, data, as_string=False):
 
         if not data:
-            return 
+            return ''
 
-        x = PrettyTable(["Application name", "Web", "status", "services"], hrules=ALL)
+        x = PrettyTable(["Application name", "Web", "status", "cpu %", "memory", "services"], hrules=ALL)
         for app in data:
 
             volume_services = {}
@@ -148,7 +149,11 @@ class ApiRpcClient(object):
                         service['ports']['22/tcp'][0]['HostPort'],
                     )
 
+            service_memory = []
+            service_cpu = []
             services = []
+            app_cpu = 0.0
+            app_mem = 0
             for service in app['services']:
                 if service['name'].startswith('_volumes_'):
                     continue
@@ -178,17 +183,28 @@ class ApiRpcClient(object):
                         data += ' (%s)' % ', '.join(service['volumes'])
 
                 services.append(data)
+                service_memory.append(str(service['memory']) + 'M')
+                service_cpu.append(("%.2f" % float(service['cpu'])) + '%')
+
+                app_mem += int(service['memory'])
+                app_cpu += float(service['cpu'])
 
             if app['running']:
                 app_status = app['status']
                 services_list = '\n'.join(services)
+                services_cpu_list = '\n'.join(service_cpu) + ('\n-----\n%.2f' % app_cpu) + '%'
+                services_memory_list = '\n'.join(service_memory) + ('\n-----\n' + str(app_mem)) + 'M'
 
             elif app['status'] == 'error':
                 app_status = 'ERROR'
                 services_list = app['message']
+                services_cpu_list = ''
+                services_memory_list = ''
             else:
                 app_status = ''
                 services_list = '\n'.join(services)
+                services_cpu_list = ''
+                services_memory_list = ''
 
 
             if app['status'] != 'error':
@@ -206,12 +222,35 @@ class ApiRpcClient(object):
             else:
                 web = ''
 
-            x.add_row([app['name'], web, app_status, services_list])
+            x.add_row([app['name'], web, app_status, services_cpu_list, services_memory_list, services_list])
 
-        print x
+        if not as_string:
+            print x
+        else:
+            return str(x)
+
+    @inlineCallbacks
+    def list(self, follow=False, **kwargs):
+        self.last_lines = 0
+
+        def _print(data):
+            ret = self.on_print_list_result(data, as_string=True)
+
+            if self.last_lines > 0:
+                print '\033[1A' * self.last_lines
+
+            print ret
+
+            self.last_lines = ret.count('\n') + 2
+
+        if follow:
+            self.stop_reactor = False
+            while follow:
+                yield self._remote_exec('list', _print)
+                yield sleep(1)
+        else:
+            yield self._remote_exec('list', _print)
 
-    def list(self, **kwargs):
-        self._remote_exec('list', self.on_print_list_result)
 
 
     def inspect(self, name, service, **kwargs):
@@ -275,6 +314,13 @@ class ApiRpcClient(object):
 
         self._remote_exec('start', self.on_print_list_result, name)
 
+    def create(self, name, **kwargs):
+
+        def on_result(data):
+            print 'result: %s' % pprintpp.pformat(data)
+
+        self._remote_exec('create', self.on_print_list_result, name)
+
     def format_domain(self, domain, ssl):
         if ssl:
             domain = 'https://%s' % domain
@@ -407,6 +453,7 @@ def populate_client_parser(subparsers):
     cmd.set_defaults(func='init')
 
     cmd = subparsers.add_parser('list', help='List registered applications')
+    cmd.add_argument('-f', '--follow', default=False, action='store_true', help='Continuously run list command')
     cmd.set_defaults(func='list')
 
     cmd = subparsers.add_parser('remove', help='Remove application')
@@ -417,6 +464,10 @@ def populate_client_parser(subparsers):
     cmd.add_argument('name', help='App name')
     cmd.set_defaults(func='start')
 
+    cmd = subparsers.add_parser('create', help='Create application containers')
+    cmd.add_argument('name', help='App name')
+    cmd.set_defaults(func='create')
+
     cmd = subparsers.add_parser('restart', help='Start application')
     cmd.add_argument('name', help='App name')
     cmd.set_defaults(func='restart')
